# Introduction

As a warehouse manager, managing and allocating resources to different tasks is one of your daily problems. You neither want your workers to wait for the trucks at the docks of your warehouse, nor the trucks to wait for unloading because these are wasted waiting time, thus avoidable operating costs for your warehouse.
Thanks to data providers that provide the Estimated Time of Arrival (ETA) of trucks to a given warehouse, you can now leverage real-time tracking data to optimize the allocation of trucks to the delivery docks of your warehouse.

# Solution Walkthrough

To get started with this solution, you can use the sample dataset already loaded in the solution. This sample contains data from 5 warehouses and dates from 2022-02-08 to 2022-07-18.

The Dataiku application is the project's starting point and comprises 4 sections. We'll not deal with the first 2 (warehouse data and correspondence data) for now but you can find all the information in the dedicated [ Dataiku Application article](article:11). 

For the study parameters, we'll choose : 
```Warehouse name  = warehouse_5```
```Day of observation = 2022-03-02```
![dku_app.png](w6lzrCbLP0QS)


Then, click the save button and you can start the webapp to edit the opened slots. You can keep the selected ones, change the existing available slots or even open a new dock. 
![input_slots_selection.png](mo1Zwnnmj61x)
Just keep in mind that it will have an impact on the optimization. Once selected, you can click the 'Submit Slots' button and return to the Dataiku application.

Now, everything is ready to launch the optimization. You can click the 'Optimize' button. Once the optimization has run, click on the dashboard to visualize the results.


## Understanding the results

All the information in the dashboard is related to the warehouse and the day specified earlier.

### Warehouse indicators

In this first tab, you'll find all the main indicators for the warehouse, starting with the number of shipments per type of product, day's delays, day's expected trucks, and day's expected shipments. 
![dashboard01.png](h2RoM8C5ozM1)

The 2 following charts present the delays with an overview of the day's slots and a detailed view per truck. Then, the docks and slots' availabilities are showing the selected slots in the WebApp.
Finally, the last visualization shows the number of available docks (in grey) and the slots where the trucks are expected to arrive (in blue):
![dashboard02.png](xkM3HyyETO1v)
As you can see, there are at least 4 slots where the trucks will have to wait, without considering the dock's compatibilities.

### Optimization of delivery slots
First, if you have selected enough opened slots and correct docks compatibility, you should have an "Optimal" as the last solver status.
On the right, you can see the different runs of the optimization solver.

Below, the reallocated shipments chart shows the difference between the initial (red) and optimized (green) situation.
In our case, we never exceed the availability of docks:
![initial_optimized_slots.png](StM949ND28m9)
Then, there are 4 reallocation charts (1 per dock) to see the reallocations proposed. A blank chart only means there's no reallocation done on this dock.
![reallocation_charts.png](uSnsS79pfAH2)

With the following chart, you can compare the effective time in the warehouse before and after optimization per type of ETA (done thanks to historical data available for that day) and, finally, the waiting time of trucks chart shows the same information per trucks:
![effective_time_in_warehouse.png](fpUY91C0h8M8)

All these charts are here to show the value and relevance of the optimization.

## Explore further

Now you've seen the initial study, you can try to explore another day (the supplied dataset contains data from 2022-02-08 to 2022-07-18), another warehouse, or even change the constraints.
There are 3 types of constraints you can edit directly in the Dataiku application or in the flow:
-  [Input_compatibility](dataset:input_compatibility) will have an effect on where the shipments are reallocated
- [input_weights](dataset:input_weights) will have an impact on how the shipments are reallocated
- [Input slots interface](web_app:dhw1YUK) will have an impact on both

Again, make sure that the optimization found a solution and displays an "Optimal" status in the second tab of the dashboard!

# Use your own data

In the input of this project, we used 3 types of data:
- Tracking data
- Warehouse data
- Correspondance data

## Tracking data
The[ trucks tracking data](dataset:trucks_tracking_data) comes from an external software that provides the Estimated Time of Arrival of the trucks to a given warehouse. It's the backbone of the solution and needs to contain the following columns:
 -  **truck_id** :  truck name in the tracking system. Generally, it's a unique identifier that can be converted in this solution.
 -  **warehouse_id** :  warehouse name in the tracking system. Generally, it's a unique identifier that can be converted to a human-friendly name in this project.
 -  **shipment_id** :  shipment name in the tracking system. Generally, it's a unique identifier that can be converted to a human-friendly name in this project.
 -  **destination_slot_start** :  Start time of the booked slot by the carrier
 -  **destination_slot_end** : (not mandatory) End time of the booked slot by the carrier 
 -  **distance_to_warehouse** :  (not mandatory) Truck's distance to the warehouse
 -  **position_ts** :  (not mandatory) timestamp of the received position
 -  **arrival_ts** :  (not mandatory) timestamp of the Estimated Time of arrival of the truck
 -  **departure_ts** :  (not mandatory) timestamp of the departure of the truck from the warehouse. This is only possible for historical data but is interesting to run scenarios and compare what has been done vs what would have been the optimized situation.
        

## Warehouse data

This is composed of 2 datasets as mentioned before:
The [Input_compatibility](dataset:input_compatibility) dataset contains the docks compatibilities and associated weights
 -  **Dock** :  Type of product concerned
 -  **Compatibility** : dock's compatibility, need to be aligned with the products shipments datasets
 -  **Open_weight** :  dock opening weight 
 -  **slot_weight** :  slot opening weight 
<br>

The [input_weights](dataset:input_weights) contains, for each compatibility, a waiting penalty weight that is used in the optimization:
 -  **Compatibility** :  Type of product concerned
 -  **waiting_penalty_weight** :  waiting penalty weight 

## Correspondence data
This is composed of 3 datasets as mentioned before:
The [products_shipments_correspondence](dataset:products_shipments_correspondance) contains the shipment ids and the associated type of product, it's used to link shipments to docks:
 -  **shipment_id** :  all the shipment ids
 -  **type_of_product** :  associated type of product, need to be aligned with the compatibility datasets
  
The [trucks_ID_correspondence](dataset:trucks_ID_correspondance) contains the truck's ids from the trucks tracking software and a meaningful name for end users:
 -  **truck_id** :  truck id from the truck tracking software
 -  **truck_name** :  meaningful name for end users
 
The [warehouse_correspondence](dataset:warehouse_correspondance) contains the warehouse ids trucks tracking software and a meaningful name for end users:
 -  **warehouse_id** :  warehouse id from the trucks tracking software
 -  **warehouse_name** :  meaningful name for end users



